@@ -5,7 +5,7 @@ import random
 import itertools
 from math import log10
 
-from . import mimodd_base, tmpfiles, samheader, pysamtools, covtools, pysam
+from . import mimodd_base, tmpfiles, samheader, pysamtools, pysam, pyvcf
 from .fisher import fisher as fisher_exact
 from .bioobj_base import Chromosome, Deletion
 
@@ -27,12 +27,17 @@ def delcall (ibams, icov, ofile = None, max_cov = None, min_size = None, window
     original_bams, ibams = ibams, []
     try:
         # open the coverage file and retrieve the sample information
-        coverage = covtools.CovReader(open (icov, 'r'))
-        samples = set(coverage.samplenames) 
+        coverage = pyvcf.open (icov, 'rb')
+        samples = set(coverage.info.sample_names) 
 
         headers = []
         for ibam in original_bams:
-            headers.append(samheader.Header.frombam(ibam))
+            try:
+                headers.append(samheader.Header.frombam(ibam))
+            except RuntimeError:
+                raise RuntimeError(
+                    'Aligned reads input file {0} does not seem to be a BAM file'
+                    .format(ibam))
         chrm_lengths = {sq['SN']: sq['LN'] for hdr in headers for sq in hdr['SQ']}
         
         # get the read groups of all input files as a flat object
@@ -56,7 +61,7 @@ def delcall (ibams, icov, ofile = None, max_cov = None, min_size = None, window
                     # the sample name from the cov file had several matches in the bam file(s)
                     # this is dealt with if deletions are to be called by sample name,
                     # but is inacceptable for calling them by read group id
-                    raise AssertionError('Ambiguous mapping of the sample names between the coverage and the bam input files.')
+                    raise RuntimeError('Ambiguous mapping of the sample names between the coverage and the bam input files.')
                 rgid_sm_mapping.update(matches)
             if rgid_sm_mapping:
                 # we're ok, proceed
@@ -98,7 +103,7 @@ def delcall (ibams, icov, ofile = None, max_cov = None, min_size = None, window
 
         if verbose:
             print('Starting prediction for the following samples')
-            print(', '.join(coverage.samplenames))
+            print(', '.join(coverage.info.sample_names))
             uncallable_samples = {rg:sm for rg,sm in rgid_sm_mapping.items() if not sizes[rg]}
             if uncallable_samples:
                 print('The following samples have been excluded from deletion calling (only Uncovered Regions will be reported for them):')
@@ -153,44 +158,30 @@ def delcall (ibams, icov, ofile = None, max_cov = None, min_size = None, window
             except:
                 pass
 
+
 def find_lowcov_regions(coviter, max_cov = None, min_size = None):
     if not max_cov:
         max_cov = 0
     if not min_size:
         min_size=100
-    # inside_deletion keeps track of the state - inside (1) an uncovered region or not (0)
-    # for every sample, initialize it to all 0
-    inside_deletion = {sample: False for sample in coviter.samplenames}
-    del_current = {}
-    debug = set()
-    for cov in coviter:
-        for sample in coviter.samplenames:
-            if cov.ss_cov[sample] <= max_cov:
-                # coverage below max_cov
-                if not inside_deletion[sample]:
-                    # record a new deletion start for the current sample
-                    del_current[sample] = [cov.chrom, cov.pos, cov.pos]
-                    inside_deletion[sample] = True
-                else:
-                    if del_current[sample][0] == cov.chrom:
-                        # extend the current deletion
-                        del_current[sample][2] = cov.pos # keep track of last known inside-del position
-                    else:
-                        # chromosome switch =>
-                        # need to close the current deletion, then start a new one
-                        if min_size <= del_current[sample][2] - del_current[sample][1]:
-                            yield sample, Deletion(*del_current[sample])
-                        del_current[sample] = [cov.chrom, cov.pos, cov.pos]
-            elif inside_deletion[sample]:
-                # reached the end of the current deletion, so close it 
-                if min_size <= del_current[sample][2] - del_current[sample][1]:
-                    yield sample, Deletion(*del_current[sample])
-                inside_deletion[sample] = False
-    # done, now check if there are still deletions to be closed
-    for sample in coviter.samplenames:
-        if inside_deletion[sample]:
-            if min_size <= del_current[sample][2] - del_current[sample][1]:
-                yield sample, Deletion(*del_current[sample])
+    contig_lengths = coviter.info.contigs
+    sample_names = coviter.info.sample_names
+    unc_current = {sample: ['', 1] for sample in sample_names}
+    for cov in coviter.cov_iter():
+        for sample in sample_names:
+            if unc_current[sample][0] != cov[0]:
+                if unc_current[sample][0] and min_size <= contig_lengths[unc_current[sample][0]] - unc_current[sample][1]:
+                    yield sample, Deletion(unc_current[sample][0], unc_current[sample][1], contig_lengths[unc_current[sample][0]])
+                unc_current[sample] = [cov[0], 1]                    
+            if cov[2][sample] > max_cov:
+                if min_size <= cov[1] - unc_current[sample][1]:
+                    yield sample, Deletion(unc_current[sample][0], unc_current[sample][1], cov[1] - 1)
+                unc_current[sample][1] = cov[1] + 1
+    # done, but need to process pending deletions
+    for sample in sample_names:
+        if unc_current[sample][0] and min_size <= contig_lengths[unc_current[sample][0]] - unc_current[sample][1]:
+            yield sample, Deletion(unc_current[sample][0], unc_current[sample][1], contig_lengths[unc_current[sample][0]])
+
 
 def sample_insert_sizes (ibams, rgid_sm_mapping, sampling_depth = 100):
     headers = []
@@ -245,18 +236,18 @@ def sample_insert_sizes (ibams, rgid_sm_mapping, sampling_depth = 100):
 def del_stats (deletion, adjacent_reads, sizes):
     without_mate = with_mate = short_seq = large_seq = 0
 
-    distances = []
+    effective_lengths = []
     for read in adjacent_reads:
         if read.is_paired and not read.is_reverse:
             if read.mate_is_unmapped:
                 without_mate += 1
             elif read.pnext >= deletion.stop:
                 with_mate +=1
-            distances.append(deletion.start-(read.pos+read.alen+1)) # deletion coordinates are 1-based, but pysam uses 0-based system
+            effective_lengths.append(deletion.stop - (read.pos+read.alen+1)) # deletion coordinates are 1-based, but pysam uses 0-based system
     if with_mate + without_mate == 0:
         return float('Nan')
     for length in sizes:
-        if length < deletion.stop - deletion.start + random.choice(distances):
+        if length < random.choice(effective_lengths):
             short_seq += 1
         else:
             large_seq += 1
@@ -272,6 +263,7 @@ def write_region(deletion, p_value, sample, rg = None, include_uncovered = False
         region_score = '.'
     else:
         return ''
+
     return '\t'.join([deletion.chromosome, 'MiModD', region_type,
                       str(deletion.start), str(deletion.stop),
                       region_score, '.', '.',
