@@ -10,12 +10,6 @@ from collections import namedtuple
 from . import config, mimodd_base, tmpfiles
 
 SamtoolsReturnValue = namedtuple('SamtoolsReturnValue', ['call', 'results', 'errors'])
-samtools_exe = os.path.normpath(
-    os.path.join(config.__file__, config.bin_path, 'samtools'))
-bcftools_exe = os.path.normpath(
-    os.path.join(config.__file__, config.bin_path, 'bcftools'))
-samtools_legacy_exe = os.path.normpath(
-    os.path.join(config.__file__, config.bin_path, 'samtools_legacy'))
 
 class Command (object):
     """Object representation of a samtools subcommand call.
@@ -44,7 +38,7 @@ def faidx (ref_genome):
     """Wrapper around samtools faidx."""
     
     call = "faidx " + ref_genome
-    call = '{0} {1}'.format(samtools_legacy_exe, call)
+    call = '{0} {1}'.format(config.samtools_legacy_exe, call)
     results, errors = (s.decode() for s in subprocess.Popen(call, shell = True, bufsize = -1, stdout = subprocess.PIPE, stderr = subprocess.PIPE).communicate())
 
     if errors:
@@ -61,15 +55,17 @@ def header (inputfile, iformat):
         call = "view -H -S " + inputfile
     elif iformat == "bam":
         call = "view -H " + inputfile
-    call = '{0} {1}'.format(samtools_legacy_exe, call)
+    call = '{0} {1}'.format(config.samtools_legacy_exe, call)
     
     results, errors = (s.decode() for s in subprocess.Popen(call, shell = True, stdout = subprocess.PIPE, stderr = subprocess.PIPE).communicate())
 
     if not results:
-        msg = 'Could not obtain header information from the {0} input file.'.format(iformat.upper())
         if errors:
+            msg = 'Could not obtain header information from the {0} input file.'.format(iformat.upper())
             msg += '\n{0} failed with: {1}'.format(call, errors)
-        raise AssertionError(msg)
+            raise RuntimeError(msg)
+        else:
+            return
     
     for line in results.strip().split('\n'):
         yield line
@@ -80,7 +76,7 @@ def index (inputfile, reindex = False):
     if os.path.exists(inputfile+'.bai') and not reindex:
         return SamtoolsReturnValue('', '', '')
     call = "index " + inputfile
-    call = '{0} {1}'.format(samtools_legacy_exe, call)
+    call = '{0} {1}'.format(config.samtools_legacy_exe, call)
     results, errors = (s.decode() for s in subprocess.Popen(call, shell = True, bufsize = -1, stdout = subprocess.PIPE, stderr = subprocess.PIPE).communicate())
     if errors:
         raise RuntimeError('{0}\nfailed with the following error from samtools index:\n{1}'.format(call, errors))
@@ -105,7 +101,7 @@ def reheader (template, inputfile, outputfile = None, verbose = False):
     if verbose:
         print ('generating new bam from {0} with new header from template {1}'.format(inputfile, template))
         
-    call = '{0} {1}'.format(samtools_legacy_exe, call)
+    call = '{0} {1}'.format(config.samtools_legacy_exe, call)
     p = subprocess.Popen(call, shell = True, stdout = stdout_pipe, stderr = subprocess.PIPE, stdin = stdin_pipe)
     if outputfile:
         results, errors = (s.decode() for s in p.communicate(input = input_bytes))
@@ -144,7 +140,7 @@ def sort (inbam, outfile = None, oformat = 'bam', maxmem = None, threads = None,
                                                               '-l {0}'.format(compression_level) if compression_level else '',
                                                               threads, maxmem, inbam, tmp_output)
     elif oformat == 'sam':
-        call = 'sort {0} -@ {1} -m {2} -o {3} {4} | {5} view -h -'.format('-n' if by_read_name else '', threads, maxmem, inbam, tmp_output, samtools_legacy_exe)
+        call = 'sort {0} -@ {1} -m {2} -o {3} {4} | {5} view -h -'.format('-n' if by_read_name else '', threads, maxmem, inbam, tmp_output, config.samtools_legacy_exe)
         # piping the bam to samtools view this way raises
         # '[bam_header_read] EOF marker is absent. The input is probably truncated.',
         # but it seems to be ok to ignore this
@@ -154,7 +150,7 @@ def sort (inbam, outfile = None, oformat = 'bam', maxmem = None, threads = None,
     if outfile:
         call = '{0} > {1}'.format(call, outfile)
         
-    call = '{0} {1}'.format(samtools_legacy_exe, call)
+    call = '{0} {1}'.format(config.samtools_legacy_exe, call)
     signal.signal(signal.SIGTERM, mimodd_base.catch_sigterm)
     try:
         # we may need to delete temporary files created by samtools
@@ -195,9 +191,9 @@ def cat (infiles, outfile, oformat, headerfile = None):
         for file in infiles:
             command_strings.append(file)
         if oformat == 'sam':
-            command_strings += ['| {0} view -h -o {1} -'.format(samtools_legacy_exe, outfile)]
+            command_strings += ['| {0} view -h -o {1} -'.format(config.samtools_legacy_exe, outfile)]
         call = ' '.join(command_strings)
-        call = '{0} {1}'.format(samtools_legacy_exe, call)
+        call = '{0} {1}'.format(config.samtools_legacy_exe, call)
         results, errors = (s.decode() for s in subprocess.Popen(call, shell = True, bufsize = -1, stdout = subprocess.PIPE, stderr = subprocess.PIPE).communicate())
         if errors:
             raise RuntimeError('{0}\nfailed with the following samtools error:\n{1}'.format(call, errors))
@@ -224,7 +220,7 @@ def view (infile, iformat, outfile, oformat):
     command_strings.append(infile)
 
     call = ' '.join(command_strings)
-    call = '{0} {1}'.format(samtools_exe, call)
+    call = '{0} {1}'.format(config.samtools_exe, call)
     if outfile:
         proc = subprocess.Popen(call, shell = True, stdout = subprocess.PIPE, stderr = subprocess.PIPE)
         results, errors = (s.decode() for s in proc.communicate())
