@@ -1,15 +1,16 @@
-"""Generate a SAM format header from NGS run metadata.
+"""An interface for working with SAM/BAM headers.
 
-Provides a Header class as an object representation of a sam header and a
-single function sam_header that generates and returns a Header instance.
-When run from the command line, parses the command line
-arguments, calls sam_header and either prints the returned header object or
-writes it to the specified output file."""
+The Header class defined here serves as an object representation of a SAM/BAM
+header.
+The middleware generator function validate_header_lines can be used to validate
+iterable text-based header sources.
+The sam_header function generates and returns a Header instance from its
+arguments and is called by the command line interface function run_as_main."""
 
 import sys
 import os
 import datetime
-import collections
+from collections import OrderedDict
 
 from . import pysamtools
 
@@ -24,20 +25,29 @@ class Header (dict):
                                          'PG', 'PI', 'PL', 'PU', 'SM')}},
                     'PG': {'required': {('ID', )},
                            'optional': {('PN', 'CL', 'PP', 'DS', 'VN')}}}
-                           
+    tag_priorities = {
+                    'HD': {'VN': 1, 'SO': 2},
+                    'SQ': {'SN': 1, 'LN': 2, 'AS': 3, 'M5': 4, 'SP': 5, 'UR': 6},
+                    'RG': {'ID': 1, 'SM': 2, 'DS': 3, 'CN': 4, 'DT': 5, 'FO': 6,
+                           'KS': 7, 'LB': 8, 'PG': 9, 'PI': 10, 'PL': 11, 'PU': 12},
+                    'PG': {'ID': 1, 'PN': 2, 'VN': 3, 'CL': 4, 'DS': 5, 'PP': 6}}
+    
     @classmethod
     def fromfile (cls, seqfile, fileformat):
-        if fileformat == 'fasta':
-            return cls.fromfasta(seqfile)
-        elif fileformat == 'vcf':
-            return cls.fromvcf(seqfile)
-        elif fileformat == 'sam':
-            return cls.fromsam(seqfile)
-        elif fileformat == 'bam':
-            return cls.frombam(seqfile)
-        else:
-            raise RuntimeError ("Cannot retrieve sequence information.")
-            
+        try:
+            if fileformat == 'fasta':
+                return cls.fromfasta(seqfile)
+            elif fileformat == 'vcf':
+                return cls.fromvcf(seqfile)
+            elif fileformat == 'sam':
+                return cls.fromsam(seqfile)
+            elif fileformat == 'bam':
+                return cls.frombam(seqfile)
+            else:
+                raise NotImplementedError()
+        except NotImplementedError:
+            raise RuntimeError ('Unsupported file format for SAM header construction: {0}'
+                                .format(fileformat))
     @classmethod
     def fromsam (cls, seqfile):
         return cls(pysamtools.header(seqfile, 'sam'))
@@ -48,18 +58,20 @@ class Header (dict):
     
     @classmethod
     def fromfasta (cls, seqfile):
-        info_dict = get_info_from_fasta(seqfile)
-        hdr = cls.__new__()
-        hdr.merge_sq(info_dict)
-        return hdr
+        raise NotImplementedError()
+        # info_dict = get_info_from_fasta(seqfile)
+        # hdr = cls.__new__()
+        # hdr.merge_sq(info_dict)
+        # return hdr
 
     @classmethod
     def fromvcf (cls, seqfile):
-        info_dict = get_info_from_vcf(seqfile)    
-        hdr = cls.__new__()
-        hdr.merge_rg(info_dict)
-        hdr.merge_sq(info_dict)
-        return hdr
+        raise NotImplementedError()
+        # info_dict = get_info_from_vcf(seqfile)    
+        # hdr = cls.__new__()
+        # hdr.merge_rg(info_dict)
+        # hdr.merge_sq(info_dict)
+        # return hdr
     
     def __init__ (self, headerlines = []):
         dict.__init__(self)
@@ -86,7 +98,7 @@ class Header (dict):
                         self['PG'].append({tag:value for tag, value in line_fields})
                     else:
                         # we don't know the record type, but samtools did not complain, so we accept it
-                        self.setdefault(line_tag[1:], []).append({tag:value for tag, value in line_fields})
+                        self.setdefault(line_tag[1:], []).append(OrderedDict((tag, value) for tag, value in line_fields))
         
     def merge_rg (self, other, treat = 'update', mapping = {}):
         # check for valid mapping
@@ -195,13 +207,13 @@ class Header (dict):
             rg['SM'] = '{0}_RG{1}'.format(rg['SM'],rg['ID'])
 
     def lines (self):
-        yield '@HD\t' + '\t'.join((':'.join((tag, value)) for tag, value in self['HD'].items()))
+        yield '@HD\t' + '\t'.join((':'.join((tag, value)) for tag, value in sorted(self['HD'].items(), key = lambda item: self.tag_priorities['HD'][item[0]])))
         for rg in self['RG']:
-            yield '@RG\t' + '\t'.join((':'.join((tag, value)) for tag, value in rg.items()))
+            yield '@RG\t' + '\t'.join((':'.join((tag, value)) for tag, value in sorted(rg.items(), key = lambda item: self.tag_priorities['RG'][item[0]])))
         for sq in self['SQ']:
-            yield '@SQ\t' + '\t'.join((':'.join((tag, str(value))) for tag, value in sq.items()))
+            yield '@SQ\t' + '\t'.join((':'.join((tag, str(value))) for tag, value in sorted(sq.items(), key = lambda item: self.tag_priorities['SQ'][item[0]])))
         for pg in self['PG']:
-            yield '@PG\t' + '\t'.join((':'.join((tag, value)) for tag, value in pg.items()))
+            yield '@PG\t' + '\t'.join((':'.join((tag, value)) for tag, value in sorted(pg.items(), key = lambda item: self.tag_priorities['PG'][item[0]])))
         std_hdr = Header()
         for unknown_record_type in (record_type for record_type in self if record_type not in std_hdr):
             for record in self[unknown_record_type]:
@@ -215,6 +227,9 @@ class Header (dict):
     def __str__ (self):
         ret = '\n'.join(self.lines())
         return ret
+
+    def copy (self):
+        return Header(self.lines())
     
 def validate_header_lines (hlines):
     ids_seen = { '@RG': set(),
@@ -260,16 +275,17 @@ def validate_header_lines (hlines):
         if include_line:
             yield line
 
-def sam_header(rg_id, rg_sm, comments = [], **optionals):
+def sam_header(rg_id, comments = [], optional_sm = False, **optionals):
     """Generates a Header instance from its arguments.
 
     All arguments must be iterables.
     For the optional argument comments, each element in the iterable
     is translated into a separate @CO header line.
     To support header generation for multi-sample sam files,
-    the remaining iterables must yield an equal number n of elements,
-    which are used to generate n @RG lines, where the ith line is generated from
-    the ith elements of all iterables.
+    the remaining iterables must yield an equal number n of elements
+    (but possibly containing None as placeholder),
+    which are used to generate n @RG lines, where the ith line is generated
+    from the ith elements of all iterables.
 		
     Required arguments:
     rg_id: list of read-group IDs
@@ -285,10 +301,12 @@ def sam_header(rg_id, rg_sm, comments = [], **optionals):
     rg_pu: list of platform units; unique identifiers
     comments: list of one-line text comments."""
 
-    optional_tags = {'rg_cn':'CN',
+    optional_tags = {'rg_sm':'SM',
+                     'rg_cn':'CN',
                      'rg_ds':'DS',
                      'rg_dt':'DT',
                      'rg_lb':'LB',
+                     'rg_pi':'PI',
                      'rg_pl':'PL',
                      'rg_pu':'PU'
                      }
@@ -299,13 +317,14 @@ def sam_header(rg_id, rg_sm, comments = [], **optionals):
     # - there cannot be more elements in any optional tag list than there are read groups
 
     rg_no = len(rg_id)
-    if rg_no != len(rg_sm):
-        raise ValueError ("Number of read-group IDs and number of sample names are unequal")
-
     if not all(rg_id):
         raise ValueError('No empty value allowed in rg_id.')
-    if not all(rg_sm):
-        raise ValueError('No empty value allowed in rg_sm.')
+    if not optional_sm:
+        if not optionals.get('rg_sm'):
+            if rg_id:
+                raise ValueError ('Sample names are required by default along with RG IDs, but none are provided.')
+        elif not all(optionals['rg_sm']):
+            raise ValueError('No empty value allowed in rg_sm.')
     
     for arg in optionals:
         if len(optionals[arg]) != rg_no:
@@ -313,17 +332,18 @@ def sam_header(rg_id, rg_sm, comments = [], **optionals):
                 'Number of elements passed in "{0}" argument does not match the number of read groups.'
                 .format(arg))
 
-            if optional_tags[arg] == 'DT':
-                # we need an extra format check for the DT tag because
-                # the Header class does not currently check it
-                try:
-                    for dstring in optionals[arg]:
+        if optional_tags[arg] == 'DT':
+            # we need an extra format check for the DT tag because
+            # the Header class does not currently check it
+            try:
+                for dstring in optionals[arg]:
+                    if dstring is not None:
                         d = datetime.datetime.strptime(dstring, "%Y-%m-%d")
-                except ValueError:
-                    raise RuntimeError('Bad date format "{0}". Expected YYYY-MM-DD'.format(dstring))
+            except ValueError:
+                raise RuntimeError('Bad date format "{0}". Expected YYYY-MM-DD'.format(dstring))
 
     header = Header()
-    header['RG'] = [{'ID': i, 'SM': s} for i, s in zip(rg_id, rg_sm)]
+    header['RG'] = [{'ID': i} for i in rg_id]
     for no, rg in enumerate(header['RG']):
         for arg in optionals:
             value = optionals[arg][no]
@@ -369,6 +389,7 @@ def run_as_main(**args):
     call_args = args.copy()
     if 'outputfile' in call_args:
         del call_args['outputfile']
+
     header = sam_header (**call_args)
 
     if 'outputfile' in args:
