@@ -11,9 +11,11 @@ groups. This function is used by the GALAXY interface of MiModD.
 import sys, os
 import signal
 import shutil
-import subprocess   # to call subprocesses
-import argparse     # to parse arguments out of sys.argv
-import gzip   # to handle compressed data
+import subprocess
+import argparse
+import gzip
+import platform
+
 from collections import Counter
 
 from . import config, mimodd_base, pysamtools, samheader, pysam, tmpfiles
@@ -62,6 +64,19 @@ def snap_call (mode, refgenome_or_indexdir, inputfiles, iformat = 'fastq', oform
     
     # snap_call_argcheck does basic argument validation and returns a flattened dictionary of all arguments for convenience
     args = snap_call_argcheck(FuncInfo(func = snap_call, innerscope = True), locals())
+
+    # --------------------- OS X 10.10 Yosemite --------------------
+    #   emergency action until bug-fix for SNAP becomes available
+    #
+    # SNAP produces corrupt multi-threaded output under Yosemite
+    # => run it with single thread (slow, but stable)
+    
+    platform_description = platform.uname()
+    if platform_description[0] == 'Darwin' and platform_description[2].split('.')[0] == '14':
+        # kernel version 14.x.x indicates Yosemite
+        args['threads'] = 1
+    # --------------------------------------------------------------
+    
     # read defaults from config.py
     if 'threads' not in args:
         args['threads'] = config.multithreading_level
@@ -149,9 +164,13 @@ def snap_call (mode, refgenome_or_indexdir, inputfiles, iformat = 'fastq', oform
                 pysamtools.view(comp_file, 'bam', inputfiles[0], 'sam')
                 if verbose:
                     print ('Decompressed input file {0} to {1}.'.format(comp_file, inputfiles[0]))
-            # .sam hard link for SAM files with wrong extension
-            elif args.get("iformat") == "sam" and (inputfiles[0].split('.')[-1] != 'sam'): # SNAP relies on the .sam suffix to use sam files, but we fool it with a hard link
-                hard_link = tmpfiles.tmp_hardlink(inputfiles[0], outfileprefix, '.sam', fallback = 'copy') # SNAP cannot use symbolic links, so if no hard link can be generated, we have to go for a copy of the file
+            elif args.get("iformat") == "sam" and (inputfiles[0].split('.')[-1] != 'sam'):
+                # .sam hard link for SAM files with wrong extension
+                # SNAP relies on the .sam suffix to use sam files,
+                # but we fool it with a hard link
+                # SNAP cannot use symbolic links, so if no hard link
+                # can be generated, we have to go for a copy of the file
+                hard_link = tmpfiles.tmp_hardlink(inputfiles[0], outfileprefix, '.sam', fallback = 'copy')
                 if verbose:
                     print ('Generated hard link {0} for input file {1}.'.format(hard_link, inputfiles[0]))
                 inputfiles[0] = hard_link
@@ -584,6 +603,18 @@ def clparse(args = None):
 def snap_index (ref_genome, index_out, seedsize = None, slack = None, ofactor = None, threads = None, verbose = False, quiet = True):
     """A simple wrapper around SNAP index."""
 
+    # --------------------- OS X 10.10 Yosemite --------------------
+    #   emergency action until bug-fix for SNAP becomes available
+    #
+    # SNAP produces corrupt multi-threaded output under Yosemite
+    # => run it with single thread (slow, but stable)
+    
+    platform_description = platform.uname()
+    if platform_description[0] == 'Darwin' and platform_description[2].split('.')[0] == '14':
+        # kernel version 14.x.x indicates Yosemite
+        threads = 1
+    # --------------------------------------------------------------
+
     if not threads:
         threads = config.multithreading_level
     if ofactor is not None:
